import socket
import sys
import ssl
import threading
import Terminal

class Transportschicht:

    def __init__(self):
        self.connected = False

    def verbindungAufbauen(self, ip, port):
        pass

    def verbindungAbbauen(self):
        pass

    def nachrichtSenden(self, msg):
        pass

class Transportschicht_TCP(Transportschicht):

    def __init__(self):
        super().__init__()
        self.clientSocket = None
        self.reader = None
        self.buffer_string = ""

    def verbindungAufbauen(self, ip, port=80):
        self.clientSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.clientSocket.connect((ip, port))
        self.connected = True
        Terminal.printTerminal(Terminal.MSGTYPE.NONE, "Verbindung aufgebaut!")
        self.reader = threading.Thread(target = self.datenEmpfangen)
        self.reader.start()
        

    def verbindungAbbauen(self):
        self.connected = False
        self.clientSocket.close()
        Terminal.printTerminal(Terminal.MSGTYPE.NONE, "Verbindung abgebaut!")

    def nachrichtSenden(self, msg):
        if not self.connected or not self.clientSocket:
            Terminal.printTerminal(Terminal.MSGTYPE.ERROR, "Kann Nachricht \"" + msg + "\" nicht senden, nicht verbunden!");
            return;
        self.clientSocket.send(msg.encode())
        Terminal.printTerminal(Terminal.MSGTYPE.OUT, str(msg))

    def datenEmpfangen(self):
        while self.connected:
            data = self.clientSocket.recv(1024)
            if not data: continue
            self.buffer_string += data.decode()
            if "\n" in self.buffer_string:
                next_start = 0
                for i in range(len(self.buffer_string)):
                    if self.buffer_string[i] == "\n":
                        Terminal.printTerminal(Terminal.MSGTYPE.IN, self.buffer_string[next_start:i+1])
                        next_start = i+1
                self.buffer_string = self.buffer_string[next_start:]
                        

class Transportschicht_TCP_TLS(Transportschicht):

    def __init__(self):
        super().__init__()
        self.clientSocket = None
        self.reader = None
        self.context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
        self.context.minimum_version = ssl.TLSVersion.TLSv1_2
        self.context.check_hostname = False
        self.context.verify_mode = ssl.CERT_NONE
        self.buffer_string = ""

    def verbindungAufbauen(self, ip, port=443):
        raw_socket = socket.create_connection((ip, port))
        self.clientSocket = self.context.wrap_socket(raw_socket, server_hostname=ip)
        self.connected = True
        Terminal.printTerminal(Terminal.MSGTYPE.NONE, "Verbindung aufgebaut!")
        self.reader = threading.Thread(target = self.datenEmpfangen)
        self.reader.start()

    def verbindungAbbauen(self):
        self.connected = False
        self.clientSocket.close()
        Terminal.printTerminal(Terminal.MSGTYPE.NONE, "Verbindung abgebaut!")

    def nachrichtSenden(self, msg):
        if not self.connected or not self.clientSocket:
            Terminal.printTerminal(Terminal.MSGTYPE.ERROR, "Kann Nachricht \"" + msg + "\" nicht senden, nicht verbunden!");
            return;
        self.clientSocket.send(msg.encode())
        Terminal.printTerminal(Terminal.MSGTYPE.OUT, str(msg))

    def datenEmpfangen(self):
        while self.connected:
            if True:
                data = self.clientSocket.recv(1024)
                if not self.connected or not data: break
                self.buffer_string += data.decode()
                if "\n" in self.buffer_string:
                    next_start = 0
                    for i in range(len(self.buffer_string)):
                        if self.buffer_string[i] == "\n":
                            Terminal.printTerminal(Terminal.MSGTYPE.IN, self.buffer_string[next_start:i+1])
                            next_start = i+1
                    self.buffer_string = self.buffer_string[next_start:]
        sys.exit()
